'use strict';

var CleanCSS = require('clean-css'),
    tarima = require('tarima');

var grunt = require('grunt'),
    path = require('path');

var cwd = process.cwd();

var _ = grunt.util._;

// override default behavior
tarima.add('css', function(params) {
  if (params.next === 'js') {
    return 'module.exports = ' + JSON.stringify(new CleanCSS().minify(params.source).styles) + ';';
  }
});

function logWriteDestFile(dest_file) {
  grunt.log.ok('File ' + path.relative(cwd, dest_file).cyan + ' created');
}

function writeFileBundle(dest, bundles, options) {
  _.each(bundles, function(views, dest_file) {
    var code = tarima.bundle(views.map(function(tpl) {
      return tpl.partial;
    }), options);

    grunt.file.write(dest_file, code);

    logWriteDestFile(dest_file);
  });
}

function writeFile(dest, partial, options) {
  var dest_file = path.join(dest, partial.params.keypath + '.' + partial.params.ext);

  var code = partial.compile(options.locals);

  var undef = code.indexOf('module.exports') === -1 && /^\s*function/.test(code);

  if (partial.params.ext === 'js' && undef) {
    code = 'module.exports = ' + code + ';';
  }

  grunt.file.write(dest_file, code);

  logWriteDestFile(dest_file);
}

module.exports = function(grunt) {
  grunt.registerMultiTask('tarima', 'Compile Tarima templates into JavaScript or plain output', function() {
    var options = this.options({
      wrapper: function(code) {
        return code + '\nmodule.exports = JST;';
      },
      locals: {},
      dest: '',
      cwd: ''
    });

    if (options.add) {
      for (var key in options.add) {
        tarima.add(key, options.add[key]);
      }

      delete options.add;
    }

    var bundles = {};

    this.files.forEach(function(f) {
      f.src.forEach(function(v) {
        var src = path.resolve(cwd, v),
            dest = path.resolve(cwd, f.dest || options.dest),
            partial = tarima.load(src, options);

        var hasExtension = path.basename(dest).indexOf('.') > -1;

        if (hasExtension) {
          if (!bundles[dest]) {
            bundles[dest] = [];
          }

          bundles[dest].push({ src: src, dest: dest, partial: partial });
        } else {
          writeFile(dest, partial, options);
        }
      });
    });

    writeFileBundle(options.dest, bundles, options);
  });
};
